package com.example;

import java.util.concurrent.TimeUnit;
import java.util.Date;
import java.text.SimpleDateFormat;

import org.infinispan.client.hotrod.RemoteCache;
import org.infinispan.client.hotrod.RemoteCacheManager;
import org.infinispan.client.hotrod.configuration.ConfigurationBuilder;
import org.infinispan.client.hotrod.MetadataValue;

public class ExpireTest {
	
	public static void main( String[] args ) throws Exception {
		new ExpireTest().invoke();
	}

	private boolean SHOW_METADATA = false; // don't set to true during the maxIdle test. because lastUsed would be updated by the get metadata operation.
	private final String KEY = "key";
	private final int NUM_ENTRIES = 100;
	private int EXPIRATION_SEC = 15;

	public void invoke() throws Exception {

		ConfigurationBuilder builder = new ConfigurationBuilder();
		builder.addServer().host("127.0.0.1").port(11222);
		RemoteCacheManager cacheManager = new RemoteCacheManager(builder.build());
		RemoteCache cache = cacheManager.getCache("default");

		// put entries
		for (int i=0; i<NUM_ENTRIES; i++) {
			// with lifespan
			//cache.put(KEY + i, new byte[1], EXPIRATION_SEC, TimeUnit.SECONDS);
			// with maxIdle
			cache.put(KEY + i, new byte[1], -1, TimeUnit.SECONDS, EXPIRATION_SEC, TimeUnit.SECONDS);
		}

		// show entry size continously
		System.out.println("# expiration sec = " + EXPIRATION_SEC);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		int counter=0;
		while (true) {
			if (SHOW_METADATA) {
				for (int j=0; j<NUM_ENTRIES; j++) {
				// lastused is updated by this get operation... so it can not be shown correct value for lastused.
					MetadataValue metadataValue = cache.getWithMetadata(KEY + j);
					if (metadataValue!=null) {
						long lifespan = metadataValue.getLifespan();  // the Lifespan if expiration with lifespan isactive for this entry
						long maxIlde = metadataValue.getMaxIdle();  // lhe max idle time in ms if active for this entry
						long creation = metadataValue.getCreated();  // the creation time (ms since 1970)
						long lastUsed = metadataValue.getLastUsed();  // the last access time (ms since 1970)
						System.out.println("lifespan=" + lifespan + ", maxIlde=" + maxIlde + ", created=" + sdf.format(new Date(creation)) + ", lastUsed=" + sdf.format(new Date(lastUsed)));
					}
				}
			}
			System.out.println( counter++ + "s, size=" + cache.size());
			Thread.sleep(1000L);
		}
	}
}
